package moderation

import (
	"context"
	"encoding/json"
	"net/http"
	"net/url"

	"github.com/google/go-querystring/query"
	"go.fifitido.net/twitch/api/types"
)

type GetBlockedTermsParams struct {
	// The ID of the broadcaster whose blocked terms you're getting.
	BroadcasterID string `url:"broadcaster_id"`

	// The ID of the broadcaster or a user that has permission to moderate the broadcaster's chat room.
	// This ID must match the user ID in the user access token.
	ModeratorID string `url:"moderator_id"`

	// The maximum number of items to return per page in the response.
	// The minimum page size is 1 item per page and the maximum is 100 items per page.
	// The default is 20.
	First *int `url:"first,omitempty"`

	// The cursor used to get the next page of results. The Pagination object in the response contains the cursor’s value.
	After *types.Cursor `url:"after,omitempty"`
}

type GetBlockedTermsResponse struct {
	// The list of blocked terms. The list is in descending order of when they were created (see the created_at timestamp).
	Data []BlockedTerm `json:"data"`

	// Contains information about the pagination in the response.
	// The object is empty if there are no more pages of results.
	// Read More: https://dev.twitch.tv/docs/api/guide#pagination
	Pagination types.Pagination `json:"pagination"`
}

// Gets the broadcaster’s list of non-private, blocked words or phrases.
// These are the terms that the broadcaster or moderator added manually or that were denied by AutoMod.
//
// Requires a user access token that includes the moderator:read:blocked_terms or moderator:manage:blocked_terms scope.
func (m *Moderation) GetBlockedTerms(ctx context.Context, params *GetBlockedTermsParams) (*GetBlockedTermsResponse, error) {
	v, _ := query.Values(params)
	endpoint := m.baseUrl.ResolveReference(&url.URL{Path: "moderation/blocked_terms", RawQuery: v.Encode()})

	req, err := http.NewRequestWithContext(ctx, http.MethodGet, endpoint.String(), nil)
	if err != nil {
		return nil, err
	}

	res, err := m.client.Do(req)
	if err != nil {
		return nil, err
	}
	defer res.Body.Close()

	var data GetBlockedTermsResponse
	if err := json.NewDecoder(res.Body).Decode(&data); err != nil {
		return nil, err
	}

	return &data, nil
}
